mod cli;
mod cpu;
mod error;
mod instructions;
mod keyboard;
mod memory;
mod types;
mod video;

use crate::cpu::Cpu;
use crate::keyboard::Keyboard;
use crate::memory::Mem;
use crate::video::Screen;

use cli::get_input;
// use cpu::CpuController;
use memory::MemHandle;
use std::io::{stdin, stdout, Result};
use std::thread::{self, sleep};
use std::time::Duration;
use termion::event::Key;
use termion::input::TermRead;
use termion::raw::IntoRawMode;
// use termion::raw::IntoRawMode;

use std::io::Write;

use termion::{self, clear, cursor, screen};

fn main() -> Result<()> {
    let mut stdout = stdout().into_raw_mode().unwrap();
    // let (cols, rows) = size()?;

    let config = get_input();

    let mut memory = Mem::new();
    let _ = memory.load_rom(&config.rom);

    write!(stdout, "{}{}", cursor::Hide, screen::ToAlternateScreen)?;

    // execute!(stdout, SetSize(64, 29), cursor::Hide, EnterAlternateScreen)?;
    // enable_raw_mode()?;

    let shared_memory = MemHandle::new(memory);
    let screen_memory = shared_memory.clone();
    let cpu_memory = shared_memory.clone();
    let keyboard_memory = shared_memory.clone();

    let keyboard = Keyboard::new(keyboard_memory);

    let (mut cpu, cpu_controller) = Cpu::new_with_control(cpu_memory);
    let _ = cpu.reset();

    thread::spawn(move || loop {
        cpu.cycle();
    });

    let mut screen = Screen::new(&config, cpu_controller, screen_memory);

    loop {
        keyboard.clear_keys(); // nasty hack until i can figure out a good way of clearing keyboard
                               // memory when no keys are pressed
        screen.draw();
        // if event::poll(std::time::Duration::from_millis(16))? {
        keyboard.read_keys();
        for key in stdin().keys() {
            match key.unwrap() {
                Key::Char('q') => {
                    write!(stdout, "{}", clear::All)?;
                    return Ok(());
                }
                _ => {}
            }
        }
        // }
        sleep(Duration::from_millis(16));
    }
}
